// due to linking errors
extern crate openssl;
// don't touch anything
extern crate diesel;
// in this block

pub mod database;
pub mod query_helper;

use diesel::migration::Migration;
use diesel::{migration, pg::PgConnection, Connection};
use diesel_migrations::{embed_migrations, EmbeddedMigrations, MigrationHarness};
use std::error::Error;
use u_lib::config::DBEnv;
use u_lib::db::generate_postgres_url;

const MIGRATIONS: EmbeddedMigrations = embed_migrations!();

fn main() -> Result<(), Box<dyn Error + Send + Sync>> {
    let action = action::parse_command_line()?;
    let dbconfig = DBEnv::load()?;

    database::setup_database().unwrap();

    let conn = PgConnection::establish(&generate_postgres_url(&dbconfig))?;
    run(action, conn)
}

fn run(action: action::Action, mut conn: PgConnection) -> migration::Result<()> {
    use action::Action::*;
    match action {
        ListPending => {
            let list = conn.pending_migrations(MIGRATIONS)?;
            if list.is_empty() {
                println!("No pending migrations.");
            }
            for mig in list {
                println!("Pending migration: {}", mig.name());
            }
        }
        MigrateUp => {
            let list = conn.run_pending_migrations(MIGRATIONS)?;
            if list.is_empty() {
                println!("No pending migrations.");
            }
            for mig in list {
                println!("Applied migration: {}", mig);
            }
        }
        MigrateDown => {
            let mig = conn.revert_last_migration(MIGRATIONS)?;
            println!("Reverted migration: {}", mig);
        }
    }
    Ok(())
}

mod action {
    pub enum Action {
        ListPending,
        MigrateUp,
        MigrateDown,
    }

    impl TryFrom<&str> for Action {
        type Error = ();

        fn try_from(value: &str) -> Result<Self, Self::Error> {
            match value {
                "" | "list" => Ok(Action::ListPending),
                "up" => Ok(Action::MigrateUp),
                "down" => Ok(Action::MigrateDown),
                _ => Err(()),
            }
        }
    }

    pub fn parse_command_line() -> Result<Action, String> {
        let action_str = std::env::args().nth(1).unwrap_or_default();
        let action = action_str.as_str().try_into().map_err(|_| {
            format!(
                "unrecognized command line argument: {} (expected 'up', 'down', 'list')",
                action_str
            )
        })?;
        Ok(action)
    }
}
