import { Component, OnInit } from '@angular/core';
import { TableComponent } from '../base-table/base-table.component';
import { AgentModel, Area } from '../../../models';
import { AssignJobDialogComponent, AgentInfoDialogComponent } from '../../dialogs';

@Component({
  selector: 'agent-table',
  templateUrl: './agent-table.component.html',
  styleUrls: ['../base-table/base-table.component.less'],
})
export class AgentComponent extends TableComponent<AgentModel> implements OnInit {
  area = 'agents' as Area
  displayedColumns = ['id', 'alias', 'username', 'hostname', 'last_active', 'actions']

  showItemDialog(id: string) {
    this.dataSource.getAgent(id).subscribe(resp => {
      const dialog = this.infoDialog.open(AgentInfoDialogComponent, {
        data: resp,
        width: '1000px',
      });

      const saveSub = dialog.componentInstance.onSave.subscribe(result => {
        this.dataSource.updateAgent(result).subscribe(_ => {
          alert('Saved')
          this.loadTableData()
        })
      })

      dialog.afterClosed().subscribe(result => {
        saveSub.unsubscribe()
        this.router.navigate(['.'], { relativeTo: this.route })
      })
    })
  }

  assignJobs(id: string) {
    const dialog = this.infoDialog.open(AssignJobDialogComponent, {
      data: id,
      width: '1000px',
    });
  }
}
