import { OnInit, Directive, Component } from '@angular/core';
import { ApiTableService } from '../../..';
import { MatTableDataSource } from '@angular/material/table';
import { MatDialog } from '@angular/material/dialog';
import { ApiModel, Area } from '../../../models';
import { ActivatedRoute, Router } from '@angular/router';

@Directive()
export abstract class TableComponent<T extends ApiModel> implements OnInit {
  abstract area: Area;
  table_data: MatTableDataSource<T> = new MatTableDataSource;
  isLoadingResults = true;

  constructor(
    public dataSource: ApiTableService,
    public infoDialog: MatDialog,
    public route: ActivatedRoute,
    public router: Router,
  ) { }

  ngOnInit() {
    this.loadTableData();
    this.route.queryParams.subscribe(params => {
      const id = params['id']
      const new_item = params['new']
      if (id) {
        this.showItemDialog(id);
      }
      if (new_item) {
        this.showItemDialog(null);
      }
    })
    //interval(10000).subscribe(_ => this.loadTableData());
  }

  loadTableData() {
    this.isLoadingResults = true;

    this.dataSource.getMany(this.area).subscribe(resp => {
      this.isLoadingResults = false;
      this.table_data.data = resp;
    })
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.table_data.filter = filterValue.trim().toLowerCase();
  }

  deleteItem(id: string) {
    if (confirm(`Delete ${id}?`)) {
      this.dataSource.delete(id, this.area).subscribe(_ => { })
      this.loadTableData()
    }
  }

  abstract displayedColumns: string[];
  abstract showItemDialog(id: string | null): void;
}