import { Component, OnInit } from '@angular/core';
import { TableComponent } from '../base-table/base-table.component';
import { Area, JobModel, Job } from '../../../models';
import { JobInfoDialogComponent } from '../../dialogs';
import { Observable } from 'rxjs';

@Component({
  selector: 'job-table',
  templateUrl: './job-table.component.html',
  styleUrls: ['../base-table/base-table.component.less'],
  providers: [{ provide: 'area', useValue: 'jobs' }]
})
export class JobComponent extends TableComponent<JobModel> {
  area = 'jobs' as Area;
  displayedColumns = ['id', 'alias', 'platform', 'schedule', 'exec_type', 'actions']

  showItemDialog(id: string | null) {
    const is_new_job = id === null;

    var dialogData$: Observable<Job>;

    if (is_new_job) {
      dialogData$ = new Observable(subscriber => {
        var defaultJob: Job = {
          meta: {
            alias: null,
            argv: '',
            exec_type: 'shell',
            target_platforms: '*',
            payload_id: null,
            schedule: null
          },
          payload: null
        };
        subscriber.next(defaultJob)
      })
    } else {
      dialogData$ = this.dataSource.getJob(id)
    }

    dialogData$.subscribe(dialogData => {
      const dialog = this.infoDialog.open(JobInfoDialogComponent, {
        data: dialogData,
        width: '1000px',
      });

      dialog.componentInstance.isPreview = !is_new_job;

      const saveSub = dialog.componentInstance.onSave.subscribe(result => {
        if (is_new_job) {
          this.dataSource.create(dialogData.meta, this.area)
            .subscribe(_ => {
              alert("Created")
              this.loadTableData()
            })
        } else {
          this.dataSource.updateJob(result)
            .subscribe(_ => {
              alert("Updated")
              this.loadTableData()
            })
        }
        dialog.close()
      })

      dialog.afterClosed().subscribe(result => {
        saveSub.unsubscribe()
        this.router.navigate(['.'], { relativeTo: this.route })
      })
    })
  }
}
