use crate::models::Job;
use crate::types::Id;
use lazy_static::lazy_static;
use parking_lot::{RwLock, RwLockReadGuard};
use std::{collections::HashMap, ops::Deref};

type Val = Job;
type Cache = HashMap<Id, Val>;

lazy_static! {
    static ref JOB_CACHE: RwLock<Cache> = RwLock::new(HashMap::new());
}

pub struct JobCache;

impl JobCache {
    pub fn insert(job: Val) {
        JOB_CACHE.write().insert(job.meta.id, job);
    }

    pub fn contains(id: Id) -> bool {
        JOB_CACHE.read().contains_key(&id)
    }

    pub fn get(id: Id) -> Option<JobCacheHolder> {
        if !Self::contains(id) {
            return None;
        }
        let lock = JOB_CACHE.read();
        Some(JobCacheHolder(lock, id))
    }

    pub fn remove(id: Id) {
        JOB_CACHE.write().remove(&id);
    }

    pub fn stats() -> Vec<Id> {
        JOB_CACHE.read().values().map(|job| job.meta.id).collect()
    }
}

pub struct JobCacheHolder(pub RwLockReadGuard<'static, Cache>, pub Id);

impl Deref for JobCacheHolder {
    type Target = Val;

    fn deref(&self) -> &Self::Target {
        self.0.get(&self.1).unwrap()
    }
}
