use std::fmt::Debug;

use anyhow::Error;

pub struct CombinedResult<T, E: Debug = Error> {
    ok: Vec<T>,
    err: Vec<E>,
}

impl<T, E: Debug> CombinedResult<T, E> {
    pub fn new() -> Self {
        Self {
            ok: vec![],
            err: vec![],
        }
    }

    pub fn push_ok(&mut self, result: T) {
        self.ok.push(result);
    }

    pub fn push_err(&mut self, err: impl Into<E>) {
        self.err.push(err.into());
    }

    pub fn unwrap(self) -> Vec<T> {
        let err_len = self.err.len();
        if err_len > 0 {
            panic!("CombinedResult has errors: {:?}", self.err);
        }
        self.ok
    }

    pub fn has_err(&self) -> bool {
        !self.err.is_empty()
    }

    pub fn unwrap_one(self) -> T {
        self.unwrap().pop().unwrap()
    }

    pub fn pop_errors(&mut self) -> Vec<E> {
        self.err.drain(..).collect()
    }
}
