use chrono::{offset::Local, DateTime};
use std::time::SystemTime;

pub fn bytes_to_string(data: &[u8]) -> String {
    String::from_utf8_lossy(data).to_string()
}

pub fn opt_to_string<T: ToString>(item: Option<T>) -> String {
    match item {
        Some(s) => s.to_string(),
        None => String::new(),
    }
}

pub fn systime_to_string(time: SystemTime) -> String {
    DateTime::<Local>::from(time)
        .format("%d/%m/%Y %T")
        .to_string()
}

pub fn bytes_to_string_truncated(data: &[u8], max_len: usize) -> String {
    if data.len() > max_len {
        let truncated = &data[..max_len];
        String::from_utf8_lossy(truncated).to_string() + " <truncated>"
    } else {
        String::from_utf8_lossy(&data).to_string()
    }
}
