use deadpool_diesel::{Manager as DManager, Pool as DPool, Runtime};
pub use diesel::pg::PgConnection;
use diesel::Connection;
use std::time::Duration;

use crate::config::DBEnv;

pub type PgAsyncPool = DPool<DManager<PgConnection>>;

pub fn generate_postgres_url(config: &DBEnv) -> String {
    format!(
        "postgres://{}:{}@{}:{}/{}",
        config.user, config.password, config.host, config.port, config.database
    )
}

pub fn async_pool(config: &DBEnv) -> PgAsyncPool {
    let db_url = generate_postgres_url(config);

    let manager = DManager::new(db_url, Runtime::Tokio1);
    DPool::builder(manager)
        .max_size(8)
        .wait_timeout(Some(Duration::from_secs(5 * 60)))
        .runtime(Runtime::Tokio1)
        .build()
        .unwrap()
}

pub fn unpooled(config: &DBEnv) -> PgConnection {
    let db_url = generate_postgres_url(config);

    PgConnection::establish(&db_url).unwrap()
}
