use std::env;
use std::io::stderr;
use std::path::Path;

use tracing_appender::rolling;
use tracing_subscriber::{fmt, prelude::*, registry, EnvFilter};

pub fn init_logger(logfile: Option<&str>) {
    if env::var("RUST_LOG").is_err() {
        env::set_var("RUST_LOG", "info")
    }

    let layer = fmt::layer().with_line_number(true);

    let output_layer = if cfg!(test) {
        layer.with_test_writer().boxed()
    } else {
        layer.with_writer(stderr).boxed()
    };

    let reg = registry()
        .with(EnvFilter::from_default_env())
        .with(output_layer);

    match logfile {
        Some(file) => {
            let file_path = Path::new(file).with_extension("log");
            reg.with(
                fmt::layer()
                    .with_writer(move || rolling::never("logs", &file_path))
                    .with_ansi(false),
            )
            .init()
        }
        None => reg.init(),
    };
}
