use crate::models::*;
use crate::types::Id;
use crate::UError;
use serde::{Deserialize, Serialize};
use std::fmt::Debug;

/// Represents types that could be used in client-server interaction
pub trait AsMsg: Clone + Serialize + Debug {}

impl AsMsg for Agent {}
impl AsMsg for AssignedJob {}
impl AsMsg for AssignedJobById {}
impl AsMsg for JobMeta {}
impl AsMsg for Reportable {}
impl AsMsg for Payload {}
impl AsMsg for RawPayload {}
impl AsMsg for Job {}
impl AsMsg for Id {}
impl AsMsg for String {}
impl AsMsg for Vec<u8> {}
impl AsMsg for () {}

impl<M: AsMsg> AsMsg for Vec<M> {}
impl<'msg, M: AsMsg> AsMsg for &'msg [M] {}

impl<M: AsMsg> AsMsg for &M {}

#[derive(Serialize, Deserialize, Clone, Debug)]
pub enum Reportable {
    Assigned(AssignedJob),
    Dummy,
    Error(UError),
}
