use serde::{Deserialize, Serialize};
use strum::Display;

#[cfg(feature = "server")]
mod server {
    pub use crate::models::schema::*;
    pub use diesel_derive_enum::DbEnum;
}
#[cfg(feature = "server")]
use self::server::*;

#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, Display)]
#[cfg_attr(
    feature = "server",
    derive(DbEnum),
    ExistingTypePath = "sql_types::Jobstate"
)]
pub enum JobState {
    /// server created a job, but client didn't get it yet
    Queued,

    // client got a job, but not running yet
    //Pending,
    /// client is currently running a job
    Running,
    Finished,
}

#[derive(Default, Serialize, Deserialize, Clone, Copy, Debug, PartialEq, Eq)]
#[serde(rename_all = "lowercase")]
#[cfg_attr(
    feature = "server",
    derive(DbEnum),
    ExistingTypePath = "sql_types::Jobtype"
)]
pub enum JobType {
    Init,
    Stats,
    #[default]
    Shell,
    Terminate,
    Update,
}
