use crate::UError;
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "lowercase")]
#[serde(tag = "status", content = "data")]
pub enum PanelResult<M> {
    Ok(M),
    Err(UError),
}

impl<M> PanelResult<M> {
    pub fn is_ok(&self) -> bool {
        matches!(self, PanelResult::Ok(_))
    }

    pub fn is_err(&self) -> bool {
        matches!(self, PanelResult::Err(_))
    }
}

impl<M: Serialize> ToString for PanelResult<M> {
    fn to_string(&self) -> String {
        serde_json::to_string(self).unwrap()
    }
}

pub type Id = uuid::Uuid;
