use daemonize::Daemonize;
use nix::{
    sys::signal::{signal, SigHandler, Signal},
    unistd::{chdir, close as fdclose, fork, getppid, setsid, ForkResult},
};
use std::process::exit;

fn _daemonize() {
    if getppid().as_raw() != 1 {
        setsig(Signal::SIGTTOU, SigHandler::SigIgn);
        setsig(Signal::SIGTTIN, SigHandler::SigIgn);
        setsig(Signal::SIGTSTP, SigHandler::SigIgn);
    }
    for fd in 0..=2 {
        match fdclose(fd) {
            _ => (),
        }
    }
    match chdir("/") {
        _ => (),
    };

    match fork() {
        Ok(ForkResult::Parent { .. }) => {
            exit(0);
        }
        Ok(ForkResult::Child) => match setsid() {
            _ => (),
        },
        Err(_) => exit(255),
    }
}

pub fn daemonize() {
    Daemonize::new().start().unwrap()
}

pub fn setsig(sig: Signal, hnd: SigHandler) {
    unsafe {
        signal(sig, hnd).unwrap();
    }
}
