import { Component } from '@angular/core';
import { Area } from 'src/app/models';
import { NewPayloadModel, PayloadModel } from 'src/app/models/payload.model';
import { PayloadInfoDialogComponent } from '../../dialogs';
import { NewPayloadDialogComponent } from '../../dialogs/new-payload-dialog/new-payload-dialog.component';
import { TableComponent } from '../base-table/base-table.component';

@Component({
  selector: 'payload-table',
  templateUrl: './payload-table.component.html',
  styleUrls: ['../base-table/base-table.component.less'],
  providers: [{ provide: 'area', useValue: 'payloads' }]
})
export class PayloadComponent extends TableComponent<PayloadModel> {
  area = 'payloads' as Area
  displayedColumns = ["name", "mime_type", "size", 'actions'];

  showItemDialog(id: string | null) {
    if (id === null) {
      const payload: NewPayloadModel = {
        name: "",
        data: []
      }
      const dialog = this.infoDialog.open(NewPayloadDialogComponent, {
        data: payload,
        width: '1000px',
      });

      dialog.componentInstance.onSave.subscribe(result => {
        this.dataSource.createPayload(result)
          .subscribe(_ => {
            alert("Created")
            this.loadTableData()
          })

        dialog.close()
      })

      dialog.afterClosed().subscribe(_ => {
        this.router.navigate(['.'], { relativeTo: this.route })
      })

    } else {
      this.dataSource.getPayload(id as string).subscribe(resp => {
        const dialog = this.infoDialog.open(PayloadInfoDialogComponent, {
          data: resp,
          width: '1000px',
        });

        const saveSub = dialog.componentInstance.onSave.subscribe(result => {
          this.dataSource.updatePayload(result)
            .subscribe(_ => {
              alert("Updated")
              this.loadTableData()
            })
          dialog.close()
        })

        dialog.afterClosed().subscribe(_ => {
          saveSub.unsubscribe()
          this.router.navigate(['.'], { relativeTo: this.route })
        })
      })
    }
  }

}
